#include "Spooler.h"
#include <unistd.h>
#include <time.h>

using namespace SafeSpool;


static unsigned buckets_forwarded=0;
static unsigned bytes_forwarded=0;
static unsigned items_forwarded=0;

class MySpooler : public Spooler {
public:
	MySpooler(const char *basename, size_t cycle_size)
	  : Spooler(basename,cycle_size)
	  {}
	
	bool forwardBatch(const Item *item, int items);
};

bool MySpooler::forwardBatch(const Item *item, int items) {
	items_forwarded += items;
	unsigned b=0;
	while (items--)
		b+=(item++)->sz;
	bytes_forwarded += b;
	buckets_forwarded++;
	//Simulate network transmit (100Mbits, 10ms rtt);
#define BYTES_PER_SEC 12500000.0
#define RTT 0.010
	double secs = b/BYTES_PER_SEC + RTT;
	long l_secs = (long)secs;
	long l_nsecs = (long)((secs-l_secs)*1000000000);
	timespec ts;
	ts.tv_sec = l_secs;
	ts.tv_nsec = l_nsecs;
	nanosleep(&ts,0);
	return true;
}


#include <stdio.h>

int main(void) {
	unsigned bytes_queued=0;
	unsigned buckets_queued=0;
	unsigned items_queued=0;

	MySpooler spl("file",10000000);
#define N 6
	Spooler::Item i[N]={
		{"hello",5},
		{"cruel",5},
		{"world",5},
		{"the quick brown fox jumped over the lazy dog",44},
		{"and then there is whole item which uses up quite a lot of space_________________________________________________________________________________________________________________",176},
		{"And some more text to fill up the file an show that bulk operation improves performance quite a lot",99}
	};
clock_t st = clock();
#define M 200000
	for(int j=0; j<M; j++) {
		spl.queueBatch(i,N);
		bytes_queued += 5+5+5+44+176+99;
		items_queued += N;
		buckets_queued++;
	}
clock_t et = clock();
double seconds=(et-st)/(double)CLOCKS_PER_SEC;
	printf("\n");
	printf("stopping\n");
	spl.stop();

printf("seconds: %.2f\n",seconds);
printf("queued   : %u bytes  %u items  %u buckets\n",bytes_queued,items_queued,buckets_queued);
printf("           %.2f bytes/sec  %.2f items/sec  %.2f buckets/sec\n",bytes_queued/seconds,items_queued/seconds,buckets_queued/seconds);

printf("forwarded: %u bytes  %u items  %u buckets\n",bytes_forwarded,items_forwarded,buckets_forwarded);
printf("           %.2f bytes/sec  %.2f items/sec  %.2f buckets/sec\n",bytes_forwarded/seconds,items_forwarded/seconds,buckets_forwarded/seconds);

printf("\nseconds: %.2f\n",seconds);
seconds=(et-st)/(double)CLOCKS_PER_SEC;
	_exit(0);
	return 0;
}
